#include "uwbgl_D3DWindowHandler9.h"
#include "uwbgl_D3DGraphicsSystem7.h"


UWBD3D_WindowHandler::UWBD3D_WindowHandler()
: m_pD3DDevice(NULL)
, m_pSwapChain(NULL)
, m_pDrawSurface(NULL)
{
}

UWBD3D_WindowHandler::~UWBD3D_WindowHandler()
{
  ShutDownHandler();
}

bool UWBD3D_WindowHandler::IsGraphicsDeviceValid()
{
  return (m_pD3DDevice != NULL && m_pSwapChain != NULL);
}

bool UWBD3D_WindowHandler::InitializeHandler(HWND hAttachedWindow)
{
  //Release connections if any already exist
  ShutDownHandler();

  // call parent class function to create graphics connection 
  if( !__super::InitializeHandler(hAttachedWindow) )
    return false;

  // Get the device from the graphics system.  Will be NULL if not yet set.
  m_pD3DDevice = UWBD3D_GraphicsSystem::GetSystem().GetD3DDevice();
  if(NULL == m_pD3DDevice)
    return false;

  m_pD3DDevice->AddRef();
  m_pSwapChain = UWBD3D_GraphicsSystem::GetSystem().CreateSwapChain(hAttachedWindow);

  return (NULL != m_pD3DDevice && NULL != m_pSwapChain);
}

void UWBD3D_WindowHandler::ShutDownHandler()
{
  if(m_pSwapChain)
  {
    m_pSwapChain->Release();
    m_pSwapChain = NULL;
  }
  if(m_pD3DDevice)
  {
    m_pD3DDevice->Release();
    m_pD3DDevice = NULL;
  }
}

void UWBD3D_WindowHandler::BeginDraw()
{
  if(NULL == m_pSwapChain || NULL == m_pD3DDevice)
    return;
  if (m_pDrawSurface )
    m_pDrawSurface->Release();
  m_pDrawSurface = NULL;
  m_pSwapChain->GetBackBuffer( 0, D3DBACKBUFFER_TYPE_MONO, &m_pDrawSurface );
  m_pD3DDevice->SetRenderTarget( 0, m_pDrawSurface );

  m_pD3DDevice->BeginScene();
}

void UWBD3D_WindowHandler::EndDrawAndShow()
{
  if(m_pD3DDevice)
    m_pD3DDevice->EndScene();

  //show the draw buffer
  if(m_pSwapChain)
    m_pSwapChain->Present(NULL, NULL, m_hAttachedWindow, NULL, NULL);

  if (m_pDrawSurface)
  {
    m_pDrawSurface->Release();
    m_pDrawSurface = NULL;
  }
}

void UWBD3D_WindowHandler::LoadW2NDCXform() const
{
  D3DXMATRIX world2ndc, toSize, toPlace;
  vec3 center = m_WCWindow.GetCenter();
  // line up the space's origins
  D3DXMatrixTranslation(&toPlace, -center.x, -center.y, 0.0);
    // scale to size 2
  D3DXMatrixScaling(&toSize, 2.0f/m_WCWindow.Width(), 2.0f/m_WCWindow.Height(), 1.0);

  D3DXMatrixMultiply(&world2ndc, &toPlace, &toSize);
  m_pD3DDevice->SetTransform(D3DTS_VIEW, &world2ndc);
}

void UWBD3D_WindowHandler::ComputeViewMatrix(D3DXMATRIX &mat) const
{   
  vec3 cam_pos = m_Camera.GetCameraPosition();
  D3DXVECTOR3 p(cam_pos.x, cam_pos.y, cam_pos.z);

  vec3 lookat = m_Camera.GetCameraLookAt();
  D3DXVECTOR3 a(lookat.x, lookat.y, lookat.z);

  vec3 upvector = m_Camera.GetCameraUpVector();
  D3DXVECTOR3 u(upvector.x, upvector.y, upvector.z);
  D3DXMatrixLookAtRH( &mat, &p, &a, &u );
   
  // Now pre-concatenate the camera X and Y rotation
  float xRot, yRot;
  vec3 xAxisRef, yAxisRef;
  D3DXMATRIX xRotMat, yRotMat, rotMat;
  m_Camera.GetMouseRotationsInRadian(xRot, xAxisRef, yRot, yAxisRef);
  // Format conversion
  D3DXVECTOR3 xRef, yRef;    // xRef and xAxis are identical!! Here we are simple doing format conversion
  xRef.x = xAxisRef.x; xRef.y = xAxisRef.y; xRef.z = xAxisRef.z;
  yRef.x = yAxisRef.x; yRef.y = yAxisRef.y; yRef.z = yAxisRef.z;
  D3DXMatrixRotationAxis(&xRotMat, &xRef, xRot);
  D3DXMatrixRotationAxis(&yRotMat, &yRef, yRot);
  D3DXMatrixMultiply(&rotMat,  &yRotMat, &xRotMat);   // My * Mx
  D3DXMatrixMultiply(&mat, &rotMat,  &mat);           // My * Mx * rotMat (Y will be rotated first)
}

void UWBD3D_WindowHandler::LoadViewXform() const
{
   D3DXMATRIX viewMat;
   ComputeViewMatrix(viewMat);
   m_pD3DDevice->SetTransform(D3DTS_VIEW, &viewMat);
}

void UWBD3D_WindowHandler::ComputeProjectionMatrix(D3DXMATRIX &mat) const
{
  float v = m_Camera.GetViewVolumeFOVInRadian();
  float n = m_Camera.GetViewVolumeNearPlane();
  float f = m_Camera.GetViewVolumeFarPlane();
  D3DXMatrixPerspectiveFovRH(&mat, v, GetAspectRatio(), n, f);

}

void UWBD3D_WindowHandler::LoadProjectionXform() const
{
  D3DXMATRIX projMat;
  ComputeProjectionMatrix(projMat);
  m_pD3DDevice->SetTransform(D3DTS_PROJECTION, &projMat);
}

void UWBD3D_WindowHandler::DeviceToWorld(int dcX, int dcY, vec3& wcPt, vec3& wcRay) const
{
  D3DXVECTOR3 ecPt, wcRayPtD3D;

  D3DXMATRIX viewMat, invView;
  ComputeViewMatrix(viewMat);
  D3DXMatrixInverse(&invView, NULL, &viewMat);  
  
  int deviceW, deviceH;
  GetDeviceSize(deviceW, deviceH);

  float fov = m_Camera.GetViewVolumeFOVInRadian();
  float tanAlpha = tanf((fov/2.0f));
  float tanBeta = (float(deviceW)/float(deviceH)) * tanAlpha;
  
  ecPt.x = ( ( (2.0f * dcX) / deviceW ) - 1 ) * tanBeta;
  ecPt.y = ( ( (2.0f * dcY) / deviceH ) - 1 ) * tanAlpha;
	ecPt.z = -1.0f; 
  D3DXVec3TransformCoord(&wcRayPtD3D, &ecPt, &invView);

  wcPt.x = invView._41;
  wcPt.y = invView._42;
  wcPt.z = invView._43;

  wcRay.x = wcRayPtD3D.x - wcPt.x;
  wcRay.y = wcRayPtD3D.y - wcPt.y;
  wcRay.z = wcRayPtD3D.z - wcPt.z;
}