// University of Washington Bothell Graphics Library
// Authors: Kelvin Sung, Steve Baer
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
///////////////////////////////////////////////////////////////////////////////////
#pragma once
#include "uwbgl_Common.h"
#include "uwbgl_Color1.h"
#include "uwbgl_Material1.h"
#include <string>

// Drawing modes
enum eShadeMode {smFlat, smGouraud};
enum eFillMode  {fmPoint, fmWireframe, fmSolid};
  
/// The draw helper provides for some basic drawing operations that can be implemented
/// with different graphic APIs. This allows simple objects to draw without any specific
/// API knowledge. This is not meant to encapsulate all drawing operations and "fancier"
/// graphics objects will proabably have to implement their own api specific draw code.
class UWB_DrawHelper
{
public:
  UWB_DrawHelper();
  virtual ~UWB_DrawHelper();

  void SetLOD(eLevelOfDetail lod);
  eLevelOfDetail GetLOD() const;

  // Subclasses return true if they support the draw operation
  // The base class just always returns false for all draw functions

  ///
  /// Draw a point at a given 3d position
  virtual bool DrawPoint( vec3 position )                 {return false;}

  ///
  /// Draw a line between a start and end point
  virtual bool DrawLine(vec3 start, vec3 end)             {return false;}

  ///
  /// Draw a circle with a given center and radius
  virtual bool DrawCircle( vec3 center, float radius)     {return false;}

  ///
  /// Draw a rectangle given two corner points
  virtual bool DrawRectangleXY( vec3 corner1, vec3 corner2) {return false;}
  virtual bool DrawRectangleXZ( vec3 corner1, vec3 corner2) {return false;}

  /// Reset all display attributes back to their
  /// default settings
  virtual void ResetAttributes();

  /// Accumulate the current model transformation with the given transformation
  /// supplied in this function's parameters
  virtual bool AccumulateModelTransform( vec3 translation,
                                         vec3 scale,
                                         mat3 rotation,
                                         vec3 pivot )     {return false;}
  
  /// Push the current model transformation on a stack of transforms.
  /// This is essentially saving a transformation state for later use
  virtual bool PushModelTransform()        {return false;}

  /// Pop the a model transformation off a stack of transforms and make current.
  /// This is essentially restoriing the saved transformation from PushModelTransform
  virtual bool PopModelTransform()         {return false;}

  ///
  /// Initialize the active model transformation to identity
  virtual bool InitializeModelTransform()  {return false;}

  /// Transform a 3d point using the current model transformation
  /// \param [in] the point to transform
  ///        [out] the transformed point
  virtual bool TransformPoint(vec3& point) {return false;}

  // Shade and Fill modes are states that the APIs usually set and forget.
  virtual void SetShadeMode( eShadeMode mode );
  eShadeMode GetShadeMode() const;
  virtual void SetFillMode( eFillMode mode );
  eFillMode GetFillMode() const;

  /// Set the size used for point drawing operations
  /// \param pt_size the size in pixels
  /// \returns the previous point size
  float SetPointSize( float pt_size );

  /// Set m_Color1
  /// \returns previous value of m_Color1
  UWB_Color SetColor1( UWB_Color color );

  /// Set m_Color2
  /// \returns previous value of m_Color2
  UWB_Color SetColor2( UWB_Color color );

  /// Set m_Material
  /// \returns previous value of m_Material
  UWB_Material SetMaterial(const UWB_Material&);

  /// Turn on/off blending. Blending makes primitive object colors
  /// blend with the existing color in the color buffer
  virtual bool EnableBlending( bool on ) {return false;}

  /// Set the current texture that will be used for mapping
  /// to  primitive (if textureing is enabled)
  /// \param texFile The path to the image file used for texture mapping
  ///        If NULL is passed, then there is no current texture
  virtual void SetTextureInfo(const wchar_t* texFile);

  /// Turn on/off texture mapping. A texture must be assigned if this is
  /// turned on in order to get texture mapping to work
  virtual bool EnableTexture(bool on) { return false; }

  /// Turn on/off lighting computation. 
  virtual bool EnableLighting(bool on) { return false; }

protected:
  eLevelOfDetail  m_lod;
  eShadeMode      m_ShadeMode;
  eFillMode       m_FillMode;

  /// Size of points to draw. Used by Point or other primitives
  /// when m_FillMode == fmPoint
  float           m_PointSize;

  /// Vertex color used by drawing functions. When
  /// flat shading, this color is used for all vertices
  UWB_Color       m_Color1;
  /// When Gouraud shading is enabled,
  /// the draw functions choose how to use the secondary color
  UWB_Color       m_Color2;

  UWB_Material    m_Material;

  ///
  /// support texture resource storage and lookup
  std::wstring     m_TexFileName;
};
