#include "uwbgl_SceneNode7.h"
#include "uwbgl_PrimitiveAxisFrame3.h"

UWB_SceneNode::UWB_SceneNode(const wchar_t* name)
: m_pPrimitive(NULL)
, m_velocity(0,0,0)
, m_bPivotVisible(false)
, m_Name(name)
{
}

UWB_SceneNode::~UWB_SceneNode()
{
  //delete the contained primitive if this node has one
  if(m_pPrimitive)
  {
    delete m_pPrimitive;
    m_pPrimitive = NULL;
  }
}

void UWB_SceneNode::Draw( eLevelOfDetail lod, UWB_DrawHelper& draw_helper ) const
{
  draw_helper.PushModelTransform();
  m_xform.SetUpModelStack( draw_helper );

  if(m_pPrimitive)
    m_pPrimitive->Draw( lod, draw_helper );

  int count = m_child_nodes.Count();
  for(int i=0; i<count; i++)
    m_child_nodes[i]->Draw( lod, draw_helper );

  if( m_bPivotVisible ) {
    m_xform.DrawPivot( draw_helper, 0.2f);
    UWB_PrimitiveAxisFrame axis;
    axis.EnableLighting(false);
    UWB_XformInfo xf;
    xf.SetScale(vec3(1.3f, 1.3f, 1.3f));
    draw_helper.PushModelTransform();
      xf.SetUpModelStack( draw_helper );
      axis.Draw(lod, draw_helper);
    draw_helper.PopModelTransform();
  }

  draw_helper.PopModelTransform();
}

void UWB_SceneNode::SetPrimitive( UWB_Primitive* pPrimitive )
{
  // check to see if a primitive already exisits
  if( m_pPrimitive )
  {
    delete m_pPrimitive;
    m_pPrimitive = NULL;
  }
  m_pPrimitive = pPrimitive;
}

UWB_Primitive* UWB_SceneNode::GetPrimitive(){ return m_pPrimitive; }
const UWB_Primitive* UWB_SceneNode::GetPrimitive() const {return m_pPrimitive;}


void UWB_SceneNode::InsertChildNode(UWB_SceneNode* pChild){ m_child_nodes.Append(pChild); }
int UWB_SceneNode::NumChildren() const { return m_child_nodes.Count(); }

UWB_SceneNode* UWB_SceneNode::GetChildNode(int index)
{
  if(index < 0 || index >= NumChildren())
    return NULL;

  return m_child_nodes[index];
}

const UWB_SceneNode* UWB_SceneNode::GetChildNode(int index) const
{
  if(index < 0 || index >= NumChildren())
    return NULL;

  return m_child_nodes[index];
}

void UWB_SceneNode::MoveNodeByVelocity(float elapsed_seconds)
{
  vec3 translate = m_xform.GetTranslation();
  translate +=  (m_velocity * elapsed_seconds);
  m_xform.SetTranslation( translate );
}

void UWB_SceneNode::SetVelocity( vec3 v )
{
  m_velocity = v;
}

vec3 UWB_SceneNode::GetVelocity() const
{
  return m_velocity;
}

UWB_XformInfo UWB_SceneNode::GetXformInfo() const
{
  return m_xform;
}

void UWB_SceneNode::SetXformInfo(const UWB_XformInfo& xform)
{
  m_xform = xform;
}

void UWB_SceneNode::SetPivotVisible( bool on )
{
  m_bPivotVisible = on;
}

bool UWB_SceneNode::IsPivotVisible() const
{
  return m_bPivotVisible;
}

UWB_BoundingBox UWB_SceneNode::GetBounds(UWB_DrawHelper& helper, bool bDraw) const
{
  UWB_BoundingBox box;
  box.MakeInvalid();
  if(m_pPrimitive)
    box.Add(m_pPrimitive->GetBoundingVolume( lodLow ) );

  helper.PushModelTransform();
  {
    m_xform.SetUpModelStack(helper);
    //Draw the box for debugging
    if(bDraw)
      box.Draw(&helper);
    vec3 minPt = box.GetMin();
    vec3 maxPt = box.GetMax();
    vec3 pt1(minPt.x, minPt.y, minPt.z);
    vec3 pt2(maxPt.x, minPt.y, minPt.z);
    vec3 pt3(maxPt.x, maxPt.y, minPt.z);
    vec3 pt4(minPt.x, maxPt.y, minPt.z);
    vec3 pt5(minPt.x, minPt.y, maxPt.z);
    vec3 pt6(maxPt.x, minPt.y, maxPt.z);
    vec3 pt7(maxPt.x, maxPt.y, maxPt.z);
    vec3 pt8(minPt.x, maxPt.y, maxPt.z);
    helper.TransformPoint( pt1 );
    helper.TransformPoint( pt2 );
    helper.TransformPoint( pt3 );
    helper.TransformPoint( pt4 );
    helper.TransformPoint( pt5 );
    helper.TransformPoint( pt6 );
    helper.TransformPoint( pt7 );
    helper.TransformPoint( pt8 );

    box.MakeInvalid();
    box.Add( UWB_BoundingBox(pt1, pt2) );
    box.Add( UWB_BoundingBox(pt3, pt4) );
    box.Add( UWB_BoundingBox(pt5, pt6) );
    box.Add( UWB_BoundingBox(pt7, pt8) );

    int count = m_child_nodes.Count();
    for(int i=0; i<count; i++)
      box.Add(m_child_nodes[i]->GetBounds(helper, bDraw));
  }
  helper.PopModelTransform();

  return box;
}

bool UWB_SceneNode::GetNodeBoundsHelper(const UWB_SceneNode* pSearchNode,
                                 UWB_BoundingBox& box,
                                 UWB_DrawHelper& helper,
                                 int level,
                                 bool bDraw) const 
{
  bool found = false;
  if( this == pSearchNode )
  {
    box = this->GetBounds( helper, bDraw );
    found = true;
  }
  else
  {
    helper.PushModelTransform();
    {
      level++;
      this->GetXformInfo().SetUpModelStack(helper);

      int count = this->NumChildren();
      for(int i=0; i<count; i++)
      {
        const UWB_SceneNode* pChildNode = this->GetChildNode( i );
        if( pChildNode->GetNodeBoundsHelper(  pSearchNode, box, helper, level, bDraw ) )
        {
          found = true;
          break;
        }
      }
      level--;
    }
    helper.PopModelTransform();
  }
  if( 0==level && found && bDraw )
  {
    helper.ResetAttributes();
    helper.SetColor1(UWB_Color(0, 0, 255));
    helper.SetShadeMode( smFlat );
    helper.SetFillMode( fmWireframe );
    box.Draw(&helper);
  }
  return found;
}

bool UWB_SceneNode::GetNodeBounds( const UWB_SceneNode* pSearchNode,
                                   UWB_BoundingBox& box,
                                   UWB_DrawHelper& helper,
                                   bool bDraw) const
{
  return GetNodeBoundsHelper(pSearchNode, box, helper, 0, bDraw );
}
